// --- Recherche des "locations forcees"

#include <stdlib.h>
#include <stdio.h>

#include <windows.h>

#include "constante.h"
#include "structure.h"
#include "aff_opti.h"
#include "affectat.h"

#include "cplex.h"
#include "calldll.h"

// Acces variables globales
extern int mcap;
extern int end, start;
extern int nb_ressources;
extern int Aff[MAX_VEH][MAX_ORDER_VEH];
extern int Loca[MAX_RES][MAX_ORDER_VEH];

extern int Ressources[MAX_RES][12];
extern int Item[MAX_RES];
extern ptmaint List_maint[MAX_RES*MAX_VEH];
extern ptint_maint List_int_maint[MAX_RES*MAX_VEH];
extern ptorder List_order[MAX_RES];

extern double max_time;		// Temps max pour Cplex

extern CPXENVptr  env ;
extern CPXLPptr   lp ;
extern int        status;
extern char     strbuf[256];
extern FILE     *solfile; // fichier des solutions detaillees

void write_aff(int, int);


// Procedure de recherche d'allocation optimale
// ============================================
// Pour une ressource donnee et un nombre de vehicules
// ===================================================

void aff_opti(int res, int nb_veh)
{
/* Declaration et allocation memoire des variables 
   contenant les donnees du probleme */

char     *probname = NULL;
int      numcols   = 0;
int      numrows   = 0;
int      objsen    = 0;
double   *obj      = NULL;
double   *rhs      = NULL;
char     *sense    = NULL;
int      *matbeg   = NULL;
int      *matcnt   = NULL;
int      *matind   = NULL;
double   *matval   = NULL;
double   *lb       = NULL;
double   *ub       = NULL;
double   *rngval   = NULL;
char     **colname = NULL;
char     *colnamestore = NULL;
char     **rowname = NULL;
char     *rownamestore = NULL;
int      colspace  = 0;
int      rowspace  = 0;
int      nzspace   = 0;
unsigned colnamespace = 0;
unsigned rownamespace = 0;

/* Variables resultats */

int      solstat;
double   objval = 0;
double   x[300]; // variables non entieres
int      *cstat = NULL;
int      *rstat = NULL;

char     *basismsg;

int i, index, j, ind;
ptorder po;


// Initialisation des locations forcees
for (i = 1; i <= MAX_ORDER_VEH; i++)
		Loca[res][i] = 0;	

// Ecriture du fichier Cplex	
write_aff(res, nb_veh);
	
// ********************
// Resolution par Cplex
// ********************
	
// Positionnement sur le fichier solution
// --------------------------------------
	 
status = CPXsetlogfile (env, solfile);
if ( status ) 
   	{
      MessageBox (NULL, "Impossible de positionner le fichier solution.\n",
                 "Erreur CPLEX", MB_OK | MB_ICONSTOP);
      goto TERMINATE;
   	}
   	
   	  
// Liberation du probleme (si necessaire) 
// --------------------------------------
     
if ( lp != NULL ) 
  	{
     status = CPXfreeprob (env, &lp);
     if ( status ) 
     	{
         sprintf (strbuf, "Erreur CPXfreeprob, Code%d.\n", status);
         MessageBox (NULL, strbuf, "Erreur CPLEX", MB_OK | MB_ICONSTOP);
      	}
   	}
   	
   	
// Creation du probleme 
// --------------------
   
lp = CPXcreateprob (env, &status, "affec");
   
   
if ( lp == NULL ) 
   	{ 
      MessageBox (NULL, "Creation de LP impossible.\n",
                  "Erreur CPLEX", MB_OK | MB_ICONSTOP);
     goto TERMINATE;
   	} 
   	
status = CPXreadcopyprob (env, lp, "affec.lp", NULL);
   	
if ( status ) 
     {
      MessageBox (NULL, "Lecture et copie du probleme impossible.\n",
                  "erreur CPLEX", MB_OK | MB_ICONSTOP);
      goto TERMINATE;
     }  
     
 // Limitation du temps
 // -------------------
 status = CPXsetdblparam (env, CPX_PARAM_TILIM, max_time);
 
 if ( status ) 
     {
     sprintf (strbuf,"Impossible de fixer la limite du temps, Status = %d\n",status);
     MessageBox (NULL, strbuf, "Erreur CPLEX", MB_OK | MB_ICONSTOP);
     goto TERMINATE;
     }
     
// Optimisation
// ------------
     
status = CPXmipopt (env, lp);
     
if ( status ) 
     {
     sprintf (strbuf,"Impossible d'optimiser, Status = %d\n",status);
     MessageBox (NULL, strbuf, "Erreur CPLEX", MB_OK | MB_ICONSTOP);
     goto TERMINATE;
     }
     
// Recuperation du resultat
// ------------------------
	
numrows = CPXgetnumrows (env, lp);
numcols = CPXgetnumcols (env, lp);

for (i=0; i<= numcols-1; i++) 
   		x[i]=0 ;
 
status = CPXgetmipx (env, lp, x,  0 , numcols-1);
 
if ( status ) 
    	{
      	MessageBox (NULL, "Impossible de recuperer les valeurs\n",
                  "erreur CPLEX", MB_OK | MB_ICONSTOP);
      	goto TERMINATE;
	   }
	   
for (i=0; i<= nb_veh; i++)
	for (j=0; j<= MAX_ORDER_VEH; j++)  
		Aff[i][j] = 0;
		
for (i=1; i<=nb_veh; i++)
	{
	po = List_order[res];
	index = 1;
	ind = 1;
	while (po != NULL)
		{
		Aff[i][index] = (int)x[(index-1)*nb_veh + i];
					
		index++;
		po = po->next;
		}
	}
   
// Branchement en cas de probleme	   
TERMINATE :
;
 	
}



// Procedure d'ecriture du fichier affec.lp
// =======================================
void write_aff(int ress, int nb_v)
{
int i, j, index, index2, compt, gv, lim, mint;
ptorder po, po2;
FILE *fich_cpl;

fich_cpl = fopen("affec.lp", "w");

fprintf(fich_cpl,"maximize \nc \nst \n");

// Ecriture de la fonction objectif
fprintf(fich_cpl,"c ");
po = List_order[ress];
index = 1;
compt = 0;
while (po != NULL)
	{
	if (po->rq > po->alloc )
		{
		for (i=1; i<=nb_v; i++)
			{
			fprintf(fich_cpl,"- %iX%i ", po->et - po->st,(index-1)*nb_v + i);
			compt++;
			if (compt >= 15)
				{
				fprintf(fich_cpl,"\n");
				compt = 0;
				}
			}
		}
	index ++;	
	po = po->next;
	}
fprintf(fich_cpl,"<= 0\n");



// Ecriture de la contrainte  Xiv <= nb_order
po = List_order[ress];
index = 1;
while (po != NULL)
	{
	if (po->rq > po->alloc + Loca[ress][index] )
		{
		for (i=1; i<=nb_v; i++)
			fprintf(fich_cpl,"+ X%i ",(index-1)*nb_v + i);
		fprintf(fich_cpl,"<= %i \n", po->rq - po->alloc - Loca[ress][index]);
		}	
	index ++;	
	po = po->next;
	}
	
// Ecriture des contraintes  (si - ej)(sj - ei) + (Xiv + Xjv -2) * GV <= 0
po = List_order[ress];
index = 1;
while (po != NULL)
	{
	if (po->rq > po->alloc + Loca[ress][index])
	   {
	   po2 = po-> next;
	   index2 = index+1;
	   while (po2 != NULL)
		{
		if (po2->rq > po2->alloc + Loca[ress][index2])
	   		{	
			for (i=1; i<=nb_v; i++)
			  {
			  if ((po->et - po->st + po2->et - po2->st) > Ressources[ress][3] *2/3) // au lieu de 2/3
				mint = Ressources[ress][4];
			  else
				mint = 0;
			  if ((po->et + mint - po2->st)*(po2->et + mint - po->st) > 0)
				{
				
				fprintf(fich_cpl,"X%i + X%i <= 1\n", (index-1)*nb_v + i,(index2-1)*nb_v + i);
				}
			  }
			}
		  po2 = po2->next;
		  index2++;
		  }
		}
	index ++;		
	po = po->next;
	}
	

// Contrainte Xi <= 1
po = List_order[ress];
index = 1;
while (po != NULL)
	{
	if (po->rq > po->alloc + Loca[ress][index])
		for (i=1; i<=nb_v; i++)
		  if (po->rq - po->alloc > 0)
			fprintf(fich_cpl," X%i <= 1 \n", (index-1)*nb_v + i);	
	index ++;
	po = po->next;
	}
	
// Contrainte Xi >=0
po = List_order[ress];
index = 1;
while (po != NULL)
	{
	if (po->rq > po->alloc + Loca[ress][index])
		for (i=1; i<=nb_v; i++)
			fprintf(fich_cpl,"X%i >=0 \n", (index-1)*nb_v + i);	
	index ++;
	po = po->next;
	}

// Integers

fprintf(fich_cpl,"\nintegers\n");

compt = 0;
po = List_order[ress];
index = 1;

while (po != NULL)
	{
	if (po->rq > po->alloc )
		for (i=1; i<=nb_v; i++)
		 	{
			fprintf(fich_cpl,"X%i ", (index-1)*nb_v + i);		
			compt++;
			if (compt >= 15)
				{
				fprintf(fich_cpl,"\n");
				compt = 0;
				}
			}
	index ++;
	po = po->next;
	}

fprintf(fich_cpl,"\nEnd\n");			

fclose(fich_cpl);

}




