// * --- main.c --- */

#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <time.h>

#include "cplex.h"
#include "calldll.h"

#include "constante.h"
#include "structure.h"
#include "main.h"
#include "read.h"
#include "affectat.h"
#include "critere.h"



// Declaration des procedures internes au module
void init();
void lib();
//void verific();
void results(int, int);


// Programme principal
// ===================
void main()
{
// Variables locales
int r, nb_vehic, v;
int i, rep, rep2;
int opt_cost;

FILE *verif;

// Indicateurs de temps d'execution 
time_t temps_debut;
time_t temps_fin;	

fich_res = fopen("results.txt", "a");

// Choix de l'execution : Avec ou sans achat
// -----------------------------------------
printf("\n Voulez vous rechercher des solutions avec achats (1) ou sans achat(0) ?");
scanf("%i", &rep);

if (rep == 0)
	{
	fprintf(fich_res, "\n\n Execution without purchase");
	max_time=300.0;
	
	}
else
	{
	fprintf(fich_res, "\n\n Execution with purchases");
	max_time = 10.0;
	}
	
// Choix de l'execution : Avec ou sans resolution par Cplex
// --------------------------------------------------------
printf("\n Voulez vous rechercher des solutions avec (1) ou sans Cplex (0)?");
scanf("%i", &rep2);


// Si resolution avec CPLEX
// ------------------------
// Initialisation de l'environnement CPLEX 
// ---------------------------------------
if (rep2 ==1)	
	{
	status = OpenCPLEXDLL();
   if (status) 
   	 { 
      sprintf (strbuf,"Probleme d'initialisation de la DLL, status = %d.\n",status);
      MessageBox (NULL, strbuf, "Erreur CPLEX", MB_OK | MB_ICONSTOP);
      exit (1);
     } 

   env = CPXopenCPLEXdevelop (&status);
   if ( env == NULL ) 
   	{
      char  errmsg[1024];
      CPXgeterrorstring (env, status, errmsg);
      sprintf (strbuf, "Could not open CPLEX environment.\n%s",errmsg);
      MessageBox (NULL, strbuf, "CPLEX Error", MB_OK | MB_ICONSTOP);
      goto TERMINATE;
   	}
   	
   solfile = cover_dll_fopen ("solution", "w");
   if (solfile == NULL ) 
    {
      MessageBox (NULL, "Ouverture du fichier solution impossible.\n",
                  "Erreur CPLEX", MB_OK | MB_ICONSTOP);
      goto TERMINATE;
   	} 
  }



// Boucle generale pour tester les differents fichiers
// --------------------------------------------------
for (i=1; i<=8; i++)
	{
	// --- Initialisations 
	// --- ===============
	// Initialisation du nombre d'item
	for (r = 1; r <= nb_ressources; r ++)
		Veh_Sup[r] = Ressources[r][12];
		
	lib();
	init();
	nb_decal = 0;
	opt_cost = MAXCOST;
	
	// Pour chaque type de ressource
	for (r = nb_ressources; r >=1 ; r--)
		// Initialisation des locations forcees
		for (v = 1; v <= MAX_ORDER_VEH; v++)
			Loca[r][v] = 0;	
	
	// --- Lecture des fichiers
	// --- ===================
	read(i, rep);

	// --- Affectation 
	// --- ===========
	temps_debut = time(NULL);

	affectation(rep2);
	
	printf(" \n RESULT : TOTAL COST = %i ", cout_total());
	fprintf(fich_res, " %i", cout_total());
				
	temps_fin = time(NULL);
	
	// --- RESULTS 
	// --- =======
	printf(" \n RESULT : TOTAL COST = %i Time %i", cout_total(), (int) difftime(temps_fin, temps_debut));
	if (cout_total() < opt_cost)
		{
		results(i, rep);
		opt_cost = cout_total();
		}
		
	fprintf(fich_res, "  %i",(int) difftime(temps_fin, temps_debut) );
	
	}
TERMINATE :
	
// Fermeture de l'environnement CPLEX
// Liberation du probleme (si necessaire) 
if (rep2 ==1)	
	{  
   if ( lp != NULL ) 
  	{
  	 printf("\n Liberation du probleme ");
     status = CPXfreeprob (env, &lp);
     if ( status ) 
     	{
         sprintf (strbuf, "Erreur CPXfreeprob, Code%d.\n", status);
         MessageBox (NULL, strbuf, "Erreur CPLEX", MB_OK | MB_ICONSTOP);
      	}
   	}
  	
// Liberation de l'environnement Cplex (si necessaire) 
   if ( env != NULL ) 
    {
       status = CPXcloseCPLEX (&env);
       printf("\n Fermeture environnement ");
   	   if ( status ) 
 	    {
         char  errmsg[1024];
         MessageBox (NULL, "Impossible de fermer l'environnement CPLEX.\n",
                     "Erreur CPLEX ", MB_OK | MB_ICONSTOP);
         CPXgeterrorstring (env, status, errmsg);
         MessageBox (NULL, errmsg, "Erreur CPLEX", MB_OK | MB_ICONSTOP);
         }
     }
     
     
// Fermeture de la DLL 
CloseCPLEXDLL();
}


fclose(fich_res);
}

// *** PROCEDURE D'INITIALISATION DES POINTEURS
// *** ========================================
void init()
{
int r, nb_vehic, v;
ptorder po;

// Initialisations de la liste d'affectation
for (r = 1; r <= nb_ressources; r ++)
	{
	nb_vehic = Ressources[r][2] + Ressources[r][12];
	for (v = 1; v <= nb_vehic; v++)
		List_affect[(r-1)*MAX_VEH + v] = NULL;
	}
	
// Initialisations de la liste des locations
for (r = 1; r <= nb_ressources; r ++)
	List_rent[r] = NULL;

// Initialisation de la liste des maintenances
for (r = 1; r <= nb_ressources; r ++)
	{
	nb_vehic = Ressources[r][2] + Ressources[r][12];
	for (v = 1; v <= nb_vehic; v++)
		List_maint[(r-1)*MAX_VEH + v] = NULL;
	}
	
// Initialisation de la liste des intervalles de maintenance
for (r = 1; r <= nb_ressources; r ++)
	{
	nb_vehic = Ressources[r][2] + Ressources[r][12];
	for (v = 1; v <= nb_vehic; v++)
		List_int_maint[(r-1)*MAX_VEH + v] = NULL;
	}
	
// Remise a 0 des allocations
for (r=1; r<=nb_ressources; r++)
	{
	if (List_order[r] != NULL)
		{
		po = List_order[r];
		while (po != NULL)
			{
			po->alloc = 0;
			po = po->next;
			}
		}
	}
	
}

// *** PROCEDURE DE LIBERATION DES POINTEURS
// *** =====================================
void lib()
{
int r, v;

ptrent pr, fr;
ptaffect pa, fa;
ptmaint pm, fm;

	// RENT
	for (r = 1; r<=nb_ressources; r ++)
		{
		if (List_rent[r] != NULL)
			{
			pr = List_rent[r];
			while (pr != NULL)
				{
				fr = pr;
				pr = pr-> next;
				free(fr);
				}
			List_rent[r] = NULL	;
			}
		}

	// ALLOC
	for (r = 1; r<=nb_ressources; r ++)
  	  {
	  for (v = 1; v <= Ressources[r][2] + Ressources[r][12] ; v++)
		{
		if (List_affect[(r-1) * MAX_VEH + v] != NULL)
			{
			pa = List_affect[(r-1) * MAX_VEH + v];
			while (pa != NULL)
				{
				fa = pa;
				pa = pa->next;
				free(fa);
				}
			List_affect[(r-1) * MAX_VEH + v] = NULL;	
			}
		  }
		}
		
	

	// MAINT
	for (r = 1; r<=nb_ressources; r ++)
	  {
	  for (v = 1; v <= Ressources[r][2] + Ressources[r][12] ; v++)
		{
		if (List_maint[(r-1) * MAX_VEH + v] != NULL)
			{
			pm = List_maint[(r-1) * MAX_VEH + v];
			while (pm != NULL)
				{
				fm = pm;
				pm = pm->next;
				free(fm);
				}
			List_maint[(r-1) * MAX_VEH + v] = NULL;
			}
		}
	  }

}


// Procedure d'affichage des resultats
// ===================================
void results(int it, int buy)
{
int r, v;
ptaffect pa;
ptrent pr;
ptmaint pm; 
FILE *fich;

if (buy == 0)
	{
	switch (it)
		{
		case 1 :
			fich=fopen("../Results/I808nb.rsl","w");
			break;
		case 2 :
			fich=fopen("../Results/I1507nb.rsl","w");
			break;
		case 3 :
			fich=fopen("../Results/I2007nb.rsl","w");
			break;
		case 4 :
			fich=fopen("../Results/I2007bisnb.rsl","w");
			break;
		case 5 :
			fich=fopen("../Results/I2109nb.rsl","w");
			break;
		case 6 :
			fich=fopen("../Results/I2109bisnb.rsl","w");
			break;
		case 7 :
			fich=fopen("../Results/I20012nb.rsl","w");
			break;
		case 8 :
			fich=fopen("../Results/I16012nb.rsl","w");
			break;	
		}
	}
	else
	{
	switch (it)
		{
		case 1 :
			fich=fopen("../Results/I808b.rsl","w");
			break;
		case 2 :
			fich=fopen("../Results/I1507b.rsl","w");
			break;
		case 3 :
			fich=fopen("../Results/I2007b.rsl","w");
			break;
		case 4 :
			fich=fopen("../Results/I2007bisb.rsl","w");
			break;
		case 5 :
			fich=fopen("../Results/I2109b.rsl","w");
			break;
		case 6 :
			fich=fopen("../Results/I2109bisb.rsl","w");
			break;
		case 7 :
			fich=fopen("../Results/I20012b.rsl","w");
			break;
		case 8 :
			fich=fopen("../Results/I16012b.rsl","w");
			break;
		}
	}
	
	
	
	fprintf(fich,"\n TOTAL COST = %i", cout_total());
		

	// RENT
	for (r = 1; r<=nb_ressources; r ++)
		{
		if (List_rent[r] != NULL)
			pr = List_rent[r];
		while (pr != NULL)
			{
			fprintf(fich,"\nrent(%i,%i,%i)", pr->rtype_id, pr->order_id, pr->r_number);
			pr = pr-> next;
			}
		}
	
	// BUY
	for (r = 1; r<=nb_ressources; r ++)
		{
		for (v = Ressources[r][2]+1; v <= Ressources[r][2] + Ressources[r][12]; v++)
			{
			if (List_affect[(r-1) * MAX_VEH + v] != NULL)
				{
				fprintf(fich,"\nbuy(%i,%i,%i)",r,(List_affect[(r-1) * MAX_VEH + v])->st,v);
				}
			}
		}
	
	// ALLOC
	for (r = 1; r<=nb_ressources; r ++)
  	  {
	  for (v = 1; v <= Ressources[r][2] + Ressources[r][12] ; v++)
		{
		if (List_affect[(r-1) * MAX_VEH + v] != NULL)
			{
			pa = List_affect[(r-1) * MAX_VEH + v];
			while (pa != NULL)
				{
				fprintf(fich,"\nalloc(%i,%i,%i)",r,(pa->order)->order_id,v);
				pa = pa->next;
				}
			}
		  }
		}
	
	// MAINT
	for (r = 1; r<=nb_ressources; r ++)
	  {
	  for (v = 1; v <= Ressources[r][2] + Ressources[r][12] ; v++)
		{
		if (List_maint[(r-1) * MAX_VEH + v] != NULL)
			{
			pm = List_maint[(r-1) * MAX_VEH + v];
			while (pm != NULL)
				{
				fprintf(fich,"\nmaint(%i,%i,%i)",r,v,pm->t);
				pm = pm->next;
				}
			}
		}
	  }
	fprintf(fich,"\n");	



fclose(fich);

}





